@@ -1,6 +1,3 @@
-mod best_numbers;
-
-use best_numbers::{best_numbers, best_numbers32};
 use eint::*;
 use proptest::prelude::*;
 
@@ -23,12 +20,12 @@ impl T64 {
 
 proptest! {
     #![proptest_config(ProptestConfig {
-        cases: 1024,
+        cases: 4096,
         ..ProptestConfig::default()
     })]
 
     #[test]
-    fn test_average_add_s(x in best_numbers(), y in best_numbers()) {
+    fn test_average_add_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::average_add_s(E64::from(x), E64::from(y));
         let r1 = Eint::average_add_s(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -36,7 +33,7 @@ proptest! {
     }
 
     #[test]
-    fn test_average_add_u(x in best_numbers(), y in best_numbers()) {
+    fn test_average_add_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::average_add_u(E64::from(x), E64::from(y));
         let r1 = Eint::average_add_u(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -44,7 +41,7 @@ proptest! {
     }
 
     #[test]
-    fn test_average_sub_s(x in best_numbers(), y in best_numbers()) {
+    fn test_average_sub_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::average_sub_s(E64::from(x), E64::from(y));
         let r1 = Eint::average_sub_s(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -52,7 +49,7 @@ proptest! {
     }
 
     #[test]
-    fn test_average_sub_u(x in best_numbers(), y in best_numbers()) {
+    fn test_average_sub_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::average_sub_u(E64::from(x), E64::from(y));
         let r1 = Eint::average_sub_u(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -60,7 +57,35 @@ proptest! {
     }
 
     #[test]
-    fn test_clz(x in best_numbers()) {
+    fn test_bit(x in 0..=u64::MAX, y in 0..=u32::MAX) {
+        let r0 = E64::from(x).bit(y);
+        let r1 = T64::recv(x).bit(y);
+        assert_eq!(r0, r1);
+        assert_eq!(r0, x.wrapping_shr(y) & 1 != 0);
+    }
+
+    #[test]
+    fn test_bit_clr(x in 0..=u64::MAX, y in 0..=u32::MAX) {
+        let mut r0 = E64::from(x);
+        let mut r1 = T64::recv(x);
+        r0.bit_clr(y);
+        r1.bit_clr(y);
+        assert_eq!(r0, r1.into());
+        assert_eq!(r0.0, x & !(1u64.wrapping_shl(y)));
+    }
+
+    #[test]
+    fn test_bit_set(x in 0..=u64::MAX, y in 0..=u32::MAX) {
+        let mut r0 = E64::from(x);
+        let mut r1 = T64::recv(x);
+        r0.bit_set(y);
+        r1.bit_set(y);
+        assert_eq!(r0, r1.into());
+        assert_eq!(r0.0, x | 1u64.wrapping_shl(y));
+    }
+
+    #[test]
+    fn test_clz(x in 0..=u64::MAX) {
         let r0 = E64::from(x).clz();
         let r1 = T64::recv(x).clz();
         assert_eq!(r0, r1);
@@ -68,7 +93,7 @@ proptest! {
     }
 
     #[test]
-    fn test_cmp_s(x in best_numbers(), y in best_numbers()) {
+    fn test_cmp_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::cmp_s(&E64::from(x), &E64::from(y));
         let r1 = Eint::cmp_s(&T64::recv(x), &T64::recv(y));
         let r2 = (x as i64).cmp(&(y as i64));
@@ -77,7 +102,7 @@ proptest! {
     }
 
     #[test]
-    fn test_cmp_u(x in best_numbers(), y in best_numbers()) {
+    fn test_cmp_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::cmp_u(&E64::from(x), &E64::from(y));
         let r1 = Eint::cmp_u(&T64::recv(x), &T64::recv(y));
         let r2 = x.cmp(&y);
@@ -86,7 +111,7 @@ proptest! {
     }
 
     #[test]
-    fn test_cpop(x in best_numbers()) {
+    fn test_cpop(x in 0..=u64::MAX) {
         let r0 = E64::from(x).cpop();
         let r1 = T64::recv(x).cpop();
         assert_eq!(r0, r1);
@@ -94,7 +119,7 @@ proptest! {
     }
 
     #[test]
-    fn test_ctz(x in best_numbers()) {
+    fn test_ctz(x in 0..=u64::MAX) {
         let r0 = E64::from(x).ctz();
         let r1 = T64::recv(x).ctz();
         assert_eq!(r0, r1);
@@ -102,14 +127,21 @@ proptest! {
     }
 
     #[test]
-    fn test_is_negative(x in best_numbers()) {
+    fn test_get(x in proptest::collection::vec(0..=u8::MAX, 32)) {
+        let r0 = E256::get(&x);
+        let r1 = unsafe { std::mem::transmute::<[u8; 32], E256>(x.as_slice().try_into().unwrap()) };
+        assert_eq!(r0, r1);
+    }
+
+    #[test]
+    fn test_is_negative(x in 0..=u64::MAX) {
         let r0 = E64::from(x).is_negative();
         let r1 = T64::recv(x).is_negative();
         assert_eq!(r0, r1);
     }
 
     #[test]
-    fn test_is_positive(x in best_numbers()) {
+    fn test_is_positive(x in 0..=u64::MAX) {
         let r0 = E64::from(x).is_positive();
         let r1 = T64::recv(x).is_positive();
         assert_eq!(r0, r1);
@@ -117,7 +149,7 @@ proptest! {
     }
 
     #[test]
-    fn test_overflowing_add_s(x in best_numbers(), y in best_numbers()) {
+    fn test_overflowing_add_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let (r0, b0) = Eint::overflowing_add_s(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::overflowing_add_s(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -125,7 +157,7 @@ proptest! {
     }
 
     #[test]
-    fn test_overflowing_add_u(x in best_numbers(), y in best_numbers()) {
+    fn test_overflowing_add_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let (r0, b0) = Eint::overflowing_add_u(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::overflowing_add_u(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -133,7 +165,7 @@ proptest! {
     }
 
     #[test]
-    fn test_overflowing_mul_s(x in best_numbers(), y in best_numbers()) {
+    fn test_overflowing_mul_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let (r0, b0) = Eint::overflowing_mul_s(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::overflowing_mul_s(T64::recv(x), T64::recv(y));
         let (r2, b2) = (x as i64).overflowing_mul(y as i64);
@@ -144,7 +176,7 @@ proptest! {
     }
 
     #[test]
-    fn test_overflowing_mul_u(x in best_numbers(), y in best_numbers()) {
+    fn test_overflowing_mul_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let (r0, b0) = Eint::overflowing_mul_u(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::overflowing_mul_u(T64::recv(x), T64::recv(y));
         let (r2, b2) = x.overflowing_mul(y);
@@ -155,7 +187,7 @@ proptest! {
     }
 
     #[test]
-    fn test_overflowing_sub_s(x in best_numbers(), y in best_numbers()) {
+    fn test_overflowing_sub_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let (r0, b0) = Eint::overflowing_sub_s(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::overflowing_sub_s(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -163,7 +195,7 @@ proptest! {
     }
 
     #[test]
-    fn test_overflowing_sub_u(x in best_numbers(), y in best_numbers()) {
+    fn test_overflowing_sub_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let (r0, b0) = Eint::overflowing_sub_u(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::overflowing_sub_u(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -171,7 +203,14 @@ proptest! {
     }
 
     #[test]
-    fn test_saturating_add_s(x in best_numbers(), y in best_numbers()){
+    fn test_put(x in proptest::collection::vec(0..=u8::MAX, 32)) {
+        let mut r = [0u8; 32];
+        E256::get(&x).put(&mut r);
+        assert_eq!(x, r);
+    }
+
+    #[test]
+    fn test_saturating_add_s(x in 0..=u64::MAX, y in 0..=u64::MAX){
         let (r0, b0) = Eint::saturating_add_s(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::saturating_add_s(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -180,7 +219,7 @@ proptest! {
     }
 
     #[test]
-    fn test_saturating_add_u(x in best_numbers(), y in best_numbers()){
+    fn test_saturating_add_u(x in 0..=u64::MAX, y in 0..=u64::MAX){
         let (r0, b0) = Eint::saturating_add_u(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::saturating_add_u(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -189,7 +228,7 @@ proptest! {
     }
 
     #[test]
-    fn test_saturating_sub_s(x in best_numbers(), y in best_numbers()){
+    fn test_saturating_sub_s(x in 0..=u64::MAX, y in 0..=u64::MAX){
         let (r0, b0) = Eint::saturating_sub_s(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::saturating_sub_s(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -198,7 +237,7 @@ proptest! {
     }
 
     #[test]
-    fn test_saturating_sub_u(x in best_numbers(), y in best_numbers()){
+    fn test_saturating_sub_u(x in 0..=u64::MAX, y in 0..=u64::MAX){
         let (r0, b0) = Eint::saturating_sub_u(E64::from(x), E64::from(y));
         let (r1, b1) = Eint::saturating_sub_u(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
@@ -207,7 +246,7 @@ proptest! {
     }
 
     #[test]
-    fn test_widdening_add_s(x in best_numbers(), y in best_numbers()) {
+    fn test_widdening_add_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::widening_add_s(E64::from(x), E64::from(y));
         let r1 = Eint::widening_add_s(T64::recv(x), T64::recv(y));
         let r2 = x as i64 as i128 + y as i64 as i128;
@@ -218,7 +257,7 @@ proptest! {
     }
 
     #[test]
-    fn test_widdening_add_u(x in best_numbers(), y in best_numbers()) {
+    fn test_widdening_add_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::widening_add_u(E64::from(x), E64::from(y));
         let r1 = Eint::widening_add_u(T64::recv(x), T64::recv(y));
         let r2 = x as u128 + y as u128;
@@ -229,7 +268,7 @@ proptest! {
     }
 
     #[test]
-    fn test_widdening_mul_s(x in best_numbers(), y in best_numbers()) {
+    fn test_widdening_mul_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::widening_mul_s(E64::from(x), E64::from(y));
         let r1 = Eint::widening_mul_s(T64::recv(x), T64::recv(y));
         let r2 = (x as i64 as i128 * y as i64 as i128) as u128;
@@ -240,7 +279,7 @@ proptest! {
     }
 
     #[test]
-    fn test_widdening_mul_su(x in best_numbers(), y in best_numbers()) {
+    fn test_widdening_mul_su(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::widening_mul_su(E64::from(x), E64::from(y));
         let r1 = Eint::widening_mul_su(T64::recv(x), T64::recv(y));
         let r2 = (x as i64 as i128 * y as u128 as i128) as u128;
@@ -251,7 +290,7 @@ proptest! {
     }
 
     #[test]
-    fn test_widdening_mul_u(x in best_numbers(), y in best_numbers()) {
+    fn test_widdening_mul_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::widening_mul_u(E64::from(x), E64::from(y));
         let r1 = Eint::widening_mul_u(T64::recv(x), T64::recv(y));
         let r2 = x as u128 * y as u128;
@@ -262,7 +301,7 @@ proptest! {
     }
 
     #[test]
-    fn test_widdening_sub_s(x in best_numbers(), y in best_numbers()) {
+    fn test_widdening_sub_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::widening_sub_s(E64::from(x), E64::from(y));
         let r1 = Eint::widening_sub_s(T64::recv(x), T64::recv(y));
         let r2 = (x as i64 as i128 - y as i64 as i128) as u128;
@@ -273,7 +312,7 @@ proptest! {
     }
 
     #[test]
-    fn test_widdening_sub_u(x in best_numbers(), y in best_numbers()) {
+    fn test_widdening_sub_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::widening_sub_u(E64::from(x), E64::from(y));
         let r1 = Eint::widening_sub_u(T64::recv(x), T64::recv(y));
         let r2 = (x as u128).wrapping_sub(y as u128);
@@ -284,14 +323,14 @@ proptest! {
     }
 
     #[test]
-    fn test_wrapping_add(x in best_numbers(), y in best_numbers()) {
+    fn test_wrapping_add(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::wrapping_add(E64::from(x), E64::from(y));
         let r1 = Eint::wrapping_add(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
     }
 
     #[test]
-    fn test_wrapping_div_s(x in best_numbers(), y in best_numbers()) {
+    fn test_wrapping_div_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::wrapping_div_s(E64::from(x), E64::from(y));
         let r1 = Eint::wrapping_div_s(T64::recv(x), T64::recv(y));
         let r2 = if y == 0 {
@@ -306,7 +345,7 @@ proptest! {
     }
 
     #[test]
-    fn test_wrapping_div_u(x in best_numbers(), y in best_numbers()) {
+    fn test_wrapping_div_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::wrapping_div_u(E64::from(x), E64::from(y));
         let r1 = Eint::wrapping_div_u(T64::recv(x), T64::recv(y));
         let r2 = if y == 0 { u64::MAX } else { x / y };
@@ -315,14 +354,14 @@ proptest! {
     }
 
     #[test]
-    fn test_wrapping_mul(x in best_numbers(), y in best_numbers()) {
+    fn test_wrapping_mul(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::wrapping_mul(E64::from(x), E64::from(y));
         let r1 = Eint::wrapping_mul(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
     }
 
     #[test]
-    fn test_wrapping_rem_s(x in best_numbers(), y in best_numbers()) {
+    fn test_wrapping_rem_s(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::wrapping_rem_s(E64::from(x), E64::from(y));
         let r1 = Eint::wrapping_rem_s(T64::recv(x), T64::recv(y));
         let r2 = if y == 0 {
@@ -337,7 +376,7 @@ proptest! {
     }
 
     #[test]
-    fn test_wrapping_rem_u(x in best_numbers(), y in best_numbers()) {
+    fn test_wrapping_rem_u(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::wrapping_rem_u(E64::from(x), E64::from(y));
         let r1 = Eint::wrapping_rem_u(T64::recv(x), T64::recv(y));
         let r2 = if y == 0 { x } else { x % y };
@@ -346,21 +385,21 @@ proptest! {
     }
 
     #[test]
-    fn test_wrapping_shl(x in best_numbers(), y in best_numbers32()) {
+    fn test_wrapping_shl(x in 0..=u64::MAX, y in 0..=u32::MAX) {
         let r0 = E64::from(x).wrapping_shl(y);
         let r1 = T64::recv(x).wrapping_shl(y);
         assert_eq!(r0, r1.into());
     }
 
     #[test]
-    fn test_wrapping_shr(x in best_numbers(), y in best_numbers32()) {
+    fn test_wrapping_shr(x in 0..=u64::MAX, y in 0..=u32::MAX) {
         let r0 = E64::from(x).wrapping_shr(y);
         let r1 = T64::recv(x).wrapping_shr(y);
         assert_eq!(r0, r1.into());
     }
 
     #[test]
-    fn test_wrapping_sra(x in best_numbers(), y in best_numbers32()) {
+    fn test_wrapping_sra(x in 0..=u64::MAX, y in 0..=u32::MAX) {
         let r0 = E64::from(x).wrapping_sra(y);
         let r1 = T64::recv(x).wrapping_sra(y);
         let r2 = (x as i64).wrapping_shr(y) as u64;
@@ -369,7 +408,7 @@ proptest! {
     }
 
     #[test]
-    fn test_wrapping_sub(x in best_numbers(), y in best_numbers()) {
+    fn test_wrapping_sub(x in 0..=u64::MAX, y in 0..=u64::MAX) {
         let r0 = Eint::wrapping_sub(E64::from(x), E64::from(y));
         let r1 = Eint::wrapping_sub(T64::recv(x), T64::recv(y));
         assert_eq!(r0, r1.into());
